package io.quarkus.registry;

import io.quarkus.registry.model.Extension;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * This exists only because ExtensionRelease does not accept a back reference to Extension
 */
@SuppressWarnings({"all"})
public final class ImmutableExtensionReleaseTuple
    implements DefaultExtensionRegistry.ExtensionReleaseTuple {
  private final Extension extension;
  private final Extension.ExtensionRelease release;

  private ImmutableExtensionReleaseTuple(
      Extension extension,
      Extension.ExtensionRelease release) {
    this.extension = extension;
    this.release = release;
  }

  /**
   * @return The value of the {@code extension} attribute
   */
  @Override
  public Extension getExtension() {
    return extension;
  }

  /**
   * @return The value of the {@code release} attribute
   */
  @Override
  public Extension.ExtensionRelease getRelease() {
    return release;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ImmutableExtensionReleaseTuple#getExtension() extension} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for extension
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableExtensionReleaseTuple withExtension(Extension value) {
    if (this.extension == value) return this;
    Extension newValue = Objects.requireNonNull(value, "extension");
    return new ImmutableExtensionReleaseTuple(newValue, this.release);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ImmutableExtensionReleaseTuple#getRelease() release} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for release
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableExtensionReleaseTuple withRelease(Extension.ExtensionRelease value) {
    if (this.release == value) return this;
    Extension.ExtensionRelease newValue = Objects.requireNonNull(value, "release");
    return new ImmutableExtensionReleaseTuple(this.extension, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableExtensionReleaseTuple} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableExtensionReleaseTuple
        && equalTo((ImmutableExtensionReleaseTuple) another);
  }

  private boolean equalTo(ImmutableExtensionReleaseTuple another) {
    return extension.equals(another.extension)
        && release.equals(another.release);
  }

  /**
   * Computes a hash code from attributes: {@code extension}, {@code release}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + extension.hashCode();
    h += (h << 5) + release.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code ExtensionReleaseTuple} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "ExtensionReleaseTuple{"
        + "extension=" + extension
        + ", release=" + release
        + "}";
  }

  /**
   * Creates an immutable copy of a {@link DefaultExtensionRegistry.ExtensionReleaseTuple} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ExtensionReleaseTuple instance
   */
  static ImmutableExtensionReleaseTuple copyOf(DefaultExtensionRegistry.ExtensionReleaseTuple instance) {
    if (instance instanceof ImmutableExtensionReleaseTuple) {
      return (ImmutableExtensionReleaseTuple) instance;
    }
    return ImmutableExtensionReleaseTuple.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableExtensionReleaseTuple ImmutableExtensionReleaseTuple}.
   * <pre>
   * ImmutableExtensionReleaseTuple.builder()
   *    .extension(io.quarkus.registry.model.Extension) // required {@link ImmutableExtensionReleaseTuple#getExtension() extension}
   *    .release(io.quarkus.registry.model.Extension.ExtensionRelease) // required {@link ImmutableExtensionReleaseTuple#getRelease() release}
   *    .build();
   * </pre>
   * @return A new ImmutableExtensionReleaseTuple builder
   */
  public static ImmutableExtensionReleaseTuple.Builder builder() {
    return new ImmutableExtensionReleaseTuple.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableExtensionReleaseTuple ImmutableExtensionReleaseTuple}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  public static final class Builder {
    private static final long INIT_BIT_EXTENSION = 0x1L;
    private static final long INIT_BIT_RELEASE = 0x2L;
    private long initBits = 0x3L;

    private Extension extension;
    private Extension.ExtensionRelease release;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code ImmutableExtensionReleaseTuple} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder from(ImmutableExtensionReleaseTuple instance) {
      return from((DefaultExtensionRegistry.ExtensionReleaseTuple) instance);
    }

    /**
     * Copy abstract value type {@code ExtensionReleaseTuple} instance into builder.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    final Builder from(DefaultExtensionRegistry.ExtensionReleaseTuple instance) {
      Objects.requireNonNull(instance, "instance");
      extension(instance.getExtension());
      release(instance.getRelease());
      return this;
    }

    /**
     * Initializes the value for the {@link ImmutableExtensionReleaseTuple#getExtension() extension} attribute.
     * @param extension The value for extension 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder extension(Extension extension) {
      this.extension = Objects.requireNonNull(extension, "extension");
      initBits &= ~INIT_BIT_EXTENSION;
      return this;
    }

    /**
     * Initializes the value for the {@link ImmutableExtensionReleaseTuple#getRelease() release} attribute.
     * @param release The value for release 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder release(Extension.ExtensionRelease release) {
      this.release = Objects.requireNonNull(release, "release");
      initBits &= ~INIT_BIT_RELEASE;
      return this;
    }

    /**
     * Builds a new {@link ImmutableExtensionReleaseTuple ImmutableExtensionReleaseTuple}.
     * @return An immutable instance of ExtensionReleaseTuple
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableExtensionReleaseTuple build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableExtensionReleaseTuple(extension, release);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = new ArrayList<>();
      if ((initBits & INIT_BIT_EXTENSION) != 0) attributes.add("extension");
      if ((initBits & INIT_BIT_RELEASE) != 0) attributes.add("release");
      return "Cannot build ExtensionReleaseTuple, some of required attributes are not set " + attributes;
    }
  }
}
