package io.quarkus.registry.builder;

import io.quarkus.registry.model.ArtifactCoords;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * Immutable implementation of {@link RegistryBuilder.ArtifactCoordsTuple}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableArtifactCoordsTuple.builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableArtifactCoordsTuple.of()}.
 */
@SuppressWarnings({"all"})
public final class ImmutableArtifactCoordsTuple
    implements RegistryBuilder.ArtifactCoordsTuple {
  private final ArtifactCoords coords;
  private final String quarkusVersion;

  private ImmutableArtifactCoordsTuple(ArtifactCoords coords, String quarkusVersion) {
    this.coords = Objects.requireNonNull(coords, "coords");
    this.quarkusVersion = Objects.requireNonNull(quarkusVersion, "quarkusVersion");
  }

  private ImmutableArtifactCoordsTuple(
      ImmutableArtifactCoordsTuple original,
      ArtifactCoords coords,
      String quarkusVersion) {
    this.coords = coords;
    this.quarkusVersion = quarkusVersion;
  }

  /**
   * @return The value of the {@code coords} attribute
   */
  @Override
  public ArtifactCoords getCoords() {
    return coords;
  }

  /**
   * @return The value of the {@code quarkusVersion} attribute
   */
  @Override
  public String getQuarkusVersion() {
    return quarkusVersion;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link RegistryBuilder.ArtifactCoordsTuple#getCoords() coords} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for coords
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableArtifactCoordsTuple withCoords(ArtifactCoords value) {
    if (this.coords == value) return this;
    ArtifactCoords newValue = Objects.requireNonNull(value, "coords");
    return new ImmutableArtifactCoordsTuple(this, newValue, this.quarkusVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link RegistryBuilder.ArtifactCoordsTuple#getQuarkusVersion() quarkusVersion} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for quarkusVersion
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableArtifactCoordsTuple withQuarkusVersion(String value) {
    String newValue = Objects.requireNonNull(value, "quarkusVersion");
    if (this.quarkusVersion.equals(newValue)) return this;
    return new ImmutableArtifactCoordsTuple(this, this.coords, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableArtifactCoordsTuple} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableArtifactCoordsTuple
        && equalTo((ImmutableArtifactCoordsTuple) another);
  }

  private boolean equalTo(ImmutableArtifactCoordsTuple another) {
    return coords.equals(another.coords)
        && quarkusVersion.equals(another.quarkusVersion);
  }

  /**
   * Computes a hash code from attributes: {@code coords}, {@code quarkusVersion}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + coords.hashCode();
    h += (h << 5) + quarkusVersion.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code ArtifactCoordsTuple} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "ArtifactCoordsTuple{"
        + "coords=" + coords
        + ", quarkusVersion=" + quarkusVersion
        + "}";
  }

  /**
   * Construct a new immutable {@code ArtifactCoordsTuple} instance.
   * @param coords The value for the {@code coords} attribute
   * @param quarkusVersion The value for the {@code quarkusVersion} attribute
   * @return An immutable ArtifactCoordsTuple instance
   */
  public static ImmutableArtifactCoordsTuple of(ArtifactCoords coords, String quarkusVersion) {
    return new ImmutableArtifactCoordsTuple(coords, quarkusVersion);
  }

  /**
   * Creates an immutable copy of a {@link RegistryBuilder.ArtifactCoordsTuple} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ArtifactCoordsTuple instance
   */
  public static ImmutableArtifactCoordsTuple copyOf(RegistryBuilder.ArtifactCoordsTuple instance) {
    if (instance instanceof ImmutableArtifactCoordsTuple) {
      return (ImmutableArtifactCoordsTuple) instance;
    }
    return ImmutableArtifactCoordsTuple.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableArtifactCoordsTuple ImmutableArtifactCoordsTuple}.
   * <pre>
   * ImmutableArtifactCoordsTuple.builder()
   *    .coords(io.quarkus.registry.model.ArtifactCoords) // required {@link RegistryBuilder.ArtifactCoordsTuple#getCoords() coords}
   *    .quarkusVersion(String) // required {@link RegistryBuilder.ArtifactCoordsTuple#getQuarkusVersion() quarkusVersion}
   *    .build();
   * </pre>
   * @return A new ImmutableArtifactCoordsTuple builder
   */
  public static ImmutableArtifactCoordsTuple.Builder builder() {
    return new ImmutableArtifactCoordsTuple.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableArtifactCoordsTuple ImmutableArtifactCoordsTuple}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  public static final class Builder {
    private static final long INIT_BIT_COORDS = 0x1L;
    private static final long INIT_BIT_QUARKUS_VERSION = 0x2L;
    private long initBits = 0x3L;

    private ArtifactCoords coords;
    private String quarkusVersion;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code ArtifactCoordsTuple} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder from(RegistryBuilder.ArtifactCoordsTuple instance) {
      Objects.requireNonNull(instance, "instance");
      coords(instance.getCoords());
      quarkusVersion(instance.getQuarkusVersion());
      return this;
    }

    /**
     * Initializes the value for the {@link RegistryBuilder.ArtifactCoordsTuple#getCoords() coords} attribute.
     * @param coords The value for coords 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder coords(ArtifactCoords coords) {
      this.coords = Objects.requireNonNull(coords, "coords");
      initBits &= ~INIT_BIT_COORDS;
      return this;
    }

    /**
     * Initializes the value for the {@link RegistryBuilder.ArtifactCoordsTuple#getQuarkusVersion() quarkusVersion} attribute.
     * @param quarkusVersion The value for quarkusVersion 
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder quarkusVersion(String quarkusVersion) {
      this.quarkusVersion = Objects.requireNonNull(quarkusVersion, "quarkusVersion");
      initBits &= ~INIT_BIT_QUARKUS_VERSION;
      return this;
    }

    /**
     * Builds a new {@link ImmutableArtifactCoordsTuple ImmutableArtifactCoordsTuple}.
     * @return An immutable instance of ArtifactCoordsTuple
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableArtifactCoordsTuple build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableArtifactCoordsTuple(null, coords, quarkusVersion);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = new ArrayList<>();
      if ((initBits & INIT_BIT_COORDS) != 0) attributes.add("coords");
      if ((initBits & INIT_BIT_QUARKUS_VERSION) != 0) attributes.add("quarkusVersion");
      return "Cannot build ArtifactCoordsTuple, some of required attributes are not set " + attributes;
    }
  }
}
