package io.quarkus.registry.model;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonUnwrapped;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * Immutable implementation of {@link Extension.ExtensionPlatformRelease}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableExtensionPlatformRelease.builder()}.
 */
@SuppressWarnings({"all"})
public final class ImmutableExtensionPlatformRelease
    implements Extension.ExtensionPlatformRelease {
  private final ArtifactCoords coords;
  private final Map<String, Object> metadata;

  private ImmutableExtensionPlatformRelease(
      ArtifactCoords coords,
      Map<String, Object> metadata) {
    this.coords = coords;
    this.metadata = metadata;
  }

  /**
   * @return The value of the {@code coords} attribute
   */
  @JsonProperty("coords")
  @JsonUnwrapped
  @Override
  public ArtifactCoords getCoords() {
    return coords;
  }

  /**
   * @return The value of the {@code metadata} attribute
   */
  @JsonProperty("metadata")
  @Override
  public Map<String, Object> getMetadata() {
    return metadata;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Extension.ExtensionPlatformRelease#getCoords() coords} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for coords
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableExtensionPlatformRelease withCoords(ArtifactCoords value) {
    if (this.coords == value) return this;
    ArtifactCoords newValue = Objects.requireNonNull(value, "coords");
    return new ImmutableExtensionPlatformRelease(newValue, this.metadata);
  }

  /**
   * Copy the current immutable object by replacing the {@link Extension.ExtensionPlatformRelease#getMetadata() metadata} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the metadata map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableExtensionPlatformRelease withMetadata(Map<String, ? extends Object> entries) {
    if (this.metadata == entries) return this;
    Map<String, Object> newValue = createUnmodifiableMap(true, false, entries);
    return new ImmutableExtensionPlatformRelease(this.coords, newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableExtensionPlatformRelease} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableExtensionPlatformRelease
        && equalTo((ImmutableExtensionPlatformRelease) another);
  }

  private boolean equalTo(ImmutableExtensionPlatformRelease another) {
    return coords.equals(another.coords);
  }

  /**
   * Computes a hash code from attributes: {@code coords}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + coords.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code ExtensionPlatformRelease} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "ExtensionPlatformRelease{"
        + "coords=" + coords
        + "}";
  }

  /**
   * Utility type used to correctly read immutable object from JSON representation.
   * @deprecated Do not use this type directly, it exists only for the <em>Jackson</em>-binding infrastructure
   */
  @Deprecated
  @JsonDeserialize
  @JsonAutoDetect(fieldVisibility = JsonAutoDetect.Visibility.NONE)
  static final class Json implements Extension.ExtensionPlatformRelease {
    ArtifactCoords coords;
    Map<String, Object> metadata = Collections.emptyMap();
    @JsonProperty("coords")
    @JsonUnwrapped
    public void setCoords(ArtifactCoords coords) {
      this.coords = coords;
    }
    @JsonProperty("metadata")
    public void setMetadata(Map<String, Object> metadata) {
      this.metadata = metadata;
    }
    @Override
    public ArtifactCoords getCoords() { throw new UnsupportedOperationException(); }
    @Override
    public Map<String, Object> getMetadata() { throw new UnsupportedOperationException(); }
  }

  /**
   * @param json A JSON-bindable data structure
   * @return An immutable value type
   * @deprecated Do not use this method directly, it exists only for the <em>Jackson</em>-binding infrastructure
   */
  @Deprecated
  @JsonCreator(mode = JsonCreator.Mode.DELEGATING)
  static ImmutableExtensionPlatformRelease fromJson(Json json) {
    ImmutableExtensionPlatformRelease.Builder builder = ImmutableExtensionPlatformRelease.builder();
    if (json.coords != null) {
      builder.coords(json.coords);
    }
    if (json.metadata != null) {
      builder.putAllMetadata(json.metadata);
    }
    return builder.build();
  }

  /**
   * Creates an immutable copy of a {@link Extension.ExtensionPlatformRelease} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ExtensionPlatformRelease instance
   */
  public static ImmutableExtensionPlatformRelease copyOf(Extension.ExtensionPlatformRelease instance) {
    if (instance instanceof ImmutableExtensionPlatformRelease) {
      return (ImmutableExtensionPlatformRelease) instance;
    }
    return ImmutableExtensionPlatformRelease.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableExtensionPlatformRelease ImmutableExtensionPlatformRelease}.
   * <pre>
   * ImmutableExtensionPlatformRelease.builder()
   *    .coords(io.quarkus.registry.model.ArtifactCoords) // required {@link Extension.ExtensionPlatformRelease#getCoords() coords}
   *    .putMetadata|putAllMetadata(String =&gt; Object) // {@link Extension.ExtensionPlatformRelease#getMetadata() metadata} mappings
   *    .build();
   * </pre>
   * @return A new ImmutableExtensionPlatformRelease builder
   */
  public static ImmutableExtensionPlatformRelease.Builder builder() {
    return new ImmutableExtensionPlatformRelease.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableExtensionPlatformRelease ImmutableExtensionPlatformRelease}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  public static final class Builder {
    private static final long INIT_BIT_COORDS = 0x1L;
    private long initBits = 0x1L;

    private ArtifactCoords coords;
    private Map<String, Object> metadata = new LinkedHashMap<String, Object>();

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code ExtensionPlatformRelease} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder from(Extension.ExtensionPlatformRelease instance) {
      Objects.requireNonNull(instance, "instance");
      coords(instance.getCoords());
      putAllMetadata(instance.getMetadata());
      return this;
    }

    /**
     * Initializes the value for the {@link Extension.ExtensionPlatformRelease#getCoords() coords} attribute.
     * @param coords The value for coords 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("coords")
    @JsonUnwrapped
    public final Builder coords(ArtifactCoords coords) {
      this.coords = Objects.requireNonNull(coords, "coords");
      initBits &= ~INIT_BIT_COORDS;
      return this;
    }

    /**
     * Put one entry to the {@link Extension.ExtensionPlatformRelease#getMetadata() metadata} map.
     * @param key The key in the metadata map
     * @param value The associated value in the metadata map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder putMetadata(String key, Object value) {
      this.metadata.put(
          Objects.requireNonNull(key, "metadata key"),
          Objects.requireNonNull(value, "metadata value"));
      return this;
    }

    /**
     * Put one entry to the {@link Extension.ExtensionPlatformRelease#getMetadata() metadata} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder putMetadata(Map.Entry<String, ? extends Object> entry) {
      String k = entry.getKey();
      Object v = entry.getValue();
      this.metadata.put(
          Objects.requireNonNull(k, "metadata key"),
          Objects.requireNonNull(v, "metadata value"));
      return this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link Extension.ExtensionPlatformRelease#getMetadata() metadata} map. Nulls are not permitted
     * @param entries The entries that will be added to the metadata map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("metadata")
    public final Builder metadata(Map<String, ? extends Object> entries) {
      this.metadata.clear();
      return putAllMetadata(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link Extension.ExtensionPlatformRelease#getMetadata() metadata} map. Nulls are not permitted
     * @param entries The entries that will be added to the metadata map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Builder putAllMetadata(Map<String, ? extends Object> entries) {
      for (Map.Entry<String, ? extends Object> e : entries.entrySet()) {
        String k = e.getKey();
        Object v = e.getValue();
        this.metadata.put(
            Objects.requireNonNull(k, "metadata key"),
            Objects.requireNonNull(v, "metadata value"));
      }
      return this;
    }

    /**
     * Builds a new {@link ImmutableExtensionPlatformRelease ImmutableExtensionPlatformRelease}.
     * @return An immutable instance of ExtensionPlatformRelease
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableExtensionPlatformRelease build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableExtensionPlatformRelease(coords, createUnmodifiableMap(false, false, metadata));
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = new ArrayList<>();
      if ((initBits & INIT_BIT_COORDS) != 0) attributes.add("coords");
      return "Cannot build ExtensionPlatformRelease, some of required attributes are not set " + attributes;
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
